import json
from pathlib import Path
from typing import Any

import typer


def get_config_file() -> Path:
    """Gets the location of the config file, as well as ensuring it exists

    Returns:
        Path: The path to the config file
    """
    # Get the config directory
    app_dir: str = typer.get_app_dir("CI-Plumber")
    # Get the config file
    config_path: Path = Path(app_dir) / "config.json"
    # If it doesn't exist, create it
    if not config_path.is_file():
        typer.echo("Config file doesn't exist yet")
        # Make the config directory
        Path.mkdir(Path(app_dir), parents=True, exist_ok=True)
        # Make the config file
        config_path.touch()
        # Fill it with valid JSON
        with config_path.open("w") as fp:
            json.dump({"repos": {}}, fp)
        typer.echo("Created config file")
    return config_path


def load_config(
    config_path: Path, remote: str
) -> tuple[dict[str, str], dict[str, dict[str, dict[str, str]]]]:
    """Loads the config file

    Args:
        config_path (Path): The path to the config file
        remote (str): The remote to use

    Returns:
        tuple[dict[str, str], dict[str, dict[str, dict[str, str]]]]: The config
            and repo data.
    """
    config: dict[str, dict[str, dict[str, str]]] = {}
    with config_path.open("r") as fp:
        config = json.load(fp)
    current_config: dict[str, str] = {}
    if remote in config["repos"]:
        current_config = config["repos"][remote]
    else:
        current_config = {}
    return current_config, config


def save_config(config_path: Path, config: dict[str, Any]) -> None:
    """Saves the config to the config file

    Args:
        config_path (Path): The path to the config file
        remote (str): The remote to use
        config (dict[str, Any]): The config to save
    """
    with config_path.open("w") as fp:
        json.dump(config, fp, indent=4)


def set_config(remote: str, key: str, value: Any) -> None:
    """Sets a config value

    Args:
        remote (str): The git remote to use
        key (str): The key for the config value
        value (Any): The value stored in the key
    """
    config_path: Path = get_config_file()
    repo_config, config = load_config(config_path, remote)
    repo_config[key] = value
    config["repos"][remote] = repo_config
    save_config(config_path, config)


def get_config(remote: str, key: str) -> Any:
    """Gets a config value

    Args:
        remote (str): The git remote to use
        key (str): The key for the config value

    Returns:
        Any: The value corresponding to the key
    """
    config_path: Path = get_config_file()
    repo_config, _ = load_config(config_path, remote)
    return repo_config[key]



import importlib
import importlib.resources
import pkgutil

import typer
from rich.console import Console
from rich.markdown import Markdown
from rich.traceback import install


from ci_plumber import docs

# Get all of the modules that have the "ci_plumber_" prefix
# https://packaging.python.org/guides/creating-and-discovering-plugins/
discovered_plugins = {
    name: importlib.import_module(name)
    for finder, name, ispkg in pkgutil.iter_modules()
    if name.startswith("ci_plumber_")
}

install(show_locals=True)


# Create the main typer app
app = typer.Typer()

# For each of the plugins found, load them as sub-commands into the main
# app
for plugin in discovered_plugins:
    # We're going a bit ambiguous here with types so I've just turned mypy
    # off
    try:
        app.add_typer(
            discovered_plugins[plugin].app,  # type: ignore
            name=discovered_plugins[plugin].name,  # type: ignore
        )
    except AttributeError:
        # It's always a possibility that there was a false positive plugin
        pass  # shhhh


@app.callback()
def callback() -> None:
    """
    CI Plumber
    """


@app.command()
def readme() -> None:
    """
    Show README.md
    """
    console = Console()
    readme = importlib.resources.read_text(docs, "README.md")
    console.print(Markdown(readme))


if __name__ == "__main__":
    app()



import subprocess
from time import sleep

from rich.console import Console


def run_command(command: str) -> str:
    """Runs a command and returns the output.

    Args:
        command (str): The command to run.

    Returns:
        str: STDOUT of the command.
    """
    sleep(2)  # Most commands seem to benefit from this delay
    console = Console()
    output = subprocess.run(command.split(), capture_output=True, text=True)

    try:
        output.check_returncode()
    except subprocess.CalledProcessError:
        console.log("[bold red]Command failed:[/bold red]")
        console.log(output.stderr)
        # console.print_exception(show_locals=True)
    return output.stdout + output.stderr



import importlib.resources
from pathlib import Path

from framework_detector import detect, get_dockerfile

from ci_plumber import templates


def generate_gitlab_yaml(
    yaml: Path = Path.cwd(),
    file_name: str = ".gitlab-ci.yml",
    overwrite: bool = False,
    template: str = "gitlab-ci.yml",
) -> None:
    """Generates the GitLab CI YAML file if it doesn't exist

    Args:
        yaml (Path): The path to the directory to generate the YAML file in
        file_name (str, optional): The nameof the YAML file to generate.
            Defaults to "gitlab-ci.yml".
    """
    if not (yaml / file_name).is_file() or overwrite:
        with (yaml / file_name).open("w") as fp:
            fp.write(importlib.resources.read_text(templates, template))


def generate_docker_file(path: Path, file_name: str = "Dockerfile") -> None:
    """Gernerates the Dockerfile if it doesn't exist

    Args:
        path (Path): The path to the directory
        file_name (str, optional): The name of the dockerfile to generate.
            Defaults to "Dockerfile".
    """
    framework = detect(path)
    dockerfile = get_dockerfile(framework["dockerfile"])

    if not (path / file_name).is_file():
        with (path / file_name).open("w") as fp:
            fp.write(dockerfile)



from ci_plumber.helpers.config_helpers import (
    get_config,
    get_config_file,
    load_config,
    save_config,
    set_config,
)
from ci_plumber.helpers.file_generators import (
    generate_docker_file,
    generate_gitlab_yaml,
)
from ci_plumber.helpers.git_helpers import get_repo
from ci_plumber.helpers.run_command import run_command



from pathlib import Path
from typing import Union

# import typer
from git import InvalidGitRepositoryError, NoSuchPathError, Repo
from git.remote import Remote
from git.util import IterableList


def get_repo(git_dir: Path = Path.cwd()) -> str:
    """Gets a repo remote from a directory

    Args:
        dir (Path): The path to the directory to get the repo from

    Returns:
        str: The repo remote
    """
    # Get the repo
    remotes: Union[IterableList[Remote], None] = None
    try:
        repo: Repo = Repo(git_dir)
        assert not repo.bare
        remotes = repo.remotes
    except (InvalidGitRepositoryError, NoSuchPathError, AssertionError):
        # typer.echo(
        #     "This directory is not a Git repository. Please run 'git init'"
        #     " first"
        # )
        ...

    # Get the remote
    remote: str = ""
    if remotes and len(remotes) == 1:
        # Get the remote
        remote = remotes[0].url
    else:
        # TODO Allow for multiple remotes
        raise KeyError("Found multiple/no remotes")
    return remote



from enum import Enum


class Module_attribute(str, Enum):
    """The attributes that a plugin can have. A valid pipeline must have one
    and only one.

    Args:
        str: This is a string enum.
        Enum: This is an enum.
    """

    source_code = "source_code"
    builder = "builder"
    image_store = "image_store"
    consumer = "consumer"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ci_plumber', 'ci_plumber.docs', 'ci_plumber.helpers', 'ci_plumber.templates']

package_data = \
{'': ['*']}

install_requires = \
['GitPython>=3.1.20,<4.0.0',
 'framework-detector>=0.2.1,<0.3.0',
 'importlib-metadata>=4.6.3,<5.0.0',
 'openshift>=0.12.1,<0.13.0',
 'python-gitlab>=2.10.0,<3.0.0',
 'rich>=10.7.0,<11.0.0',
 'typer[all]>=0.3.2,<0.4.0',
 'types-requests>=2.25.2,<3.0.0']

extras_require = \
{'all': ['ci-plumber-openshift>=0.2.0,<0.3.0',
         'ci-plumber-gitlab>=0.1.0,<0.2.0',
         'ci-plumber-azure>=0.2.0,<0.3.0']}

entry_points = \
{'console_scripts': ['ci-plumber = ci_plumber.main:app']}

setup_kwargs = {
    'name': 'ci-plumber',
    'version': '1.4.1',
    'description': 'Plumb together different CI/CD services',
    'long_description': '# CI Plumber\n\n[![CodeFactor](https://www.codefactor.io/repository/github/pbexe/ci-plumber/badge)](https://www.codefactor.io/repository/github/pbexe/ci-plumber) ![PyPI - Python Version](https://img.shields.io/pypi/pyversions/ci-plumber) ![PyPI](https://img.shields.io/pypi/v/ci-plumber) ![PyPI - Downloads](https://img.shields.io/pypi/dm/ci-plumber) [![python-app](https://github.com/pbexe/ci-plumber/actions/workflows/python-app.yml/badge.svg)](https://github.com/pbexe/ci-plumber/actions/workflows/python-app.yml)\n\nA tool to create and configure all of the stages of a CI/CD pipeline.\n\nCurrent integrations:\n- Gitlab\n- Gitlab pipelines\n- Azure App Service\n- Azure Image Registry\n- Azure MariaDB\n- Openshift\n- Openshift MariaDB\n\nFull documentation is available [here](https://milesbudden.com/ci-plumber/).\n\n## Installation\n\n```sh\npip install ci-plumber[all]\n```\n\n### Requirements\n\n- `oc` CLI tool\n- `az` CLI tool\n\n## Usage\n\n### GitLab\n```sh\n# Initialise the project\nci-plumber gitlab init\n```\n\n### OpenShift\n\n```sh\n# Deploy from the current docker registry to OpenShift\nci-plumber openshift deploy\n\n# Create a new DB and store the credentials in maria.env\nci-plumber openshift create-db\n```\n\n### Azure\n\n```sh\n# Log in to Azure\nci-plumber azure login\n\n# List your Azure subscriptions\nci-plumber azure list-subscriptions\n\n# Set the subscription to use\nci-plumber azure set-default-subscription\n\n# Create a docker registry\nci-plumber azure create-registry\n\n# Trigger a build and push\ngit add .\ngit commit -m "Added Azure CI file"\ngit tag -a v1.0.0 -m "Version 1.0.0"\ngit push --follow-tags\n\n# Deploy to Azure\nci-plumber azure create-app\n\n# Create a database and store the credentials in maria.env\nci-plumber azure create-db\n```\n\n## Developing\n\n### Installation\n```sh\n# Install dependencies\n$ poetry install\n$ poetry shell\n\n# Install git hooks\n$ pre-commit install\n$ pre-commit autoupdate\n$ pre-commit run --all-files\n\n# Symlink the plugins back to the main project\n$ ln -s ./plugins/example/ci_plumber_example/ ./ci_plumber_example\n$ ln -s ./plugins/gitlab/ci_plumber_gitlab/ ./ci_plumber_gitlab\n$ ln -s ./plugins/openshift/ci_plumber_openshift/ ./ci_plumber_openshift\n$ ln -s ./plugins/azure/ci_plumber_azure/ ./ci_plumber_azure\n```\n\n### Features\n\n- Runs checks on commit\n    - Flake8\n    - Black\n    - pre-commit-hooks checks\n    - mypy\n    - isort\n- Installable as a script\n',
    'author': 'Miles Budden',
    'author_email': 'git@miles.so',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pbexe/ci-plumber',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



# Programmatically get module version so that it is always in sync with poetry.
try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata  # type: ignore

__version__ = importlib_metadata.version(__name__)

from ci_plumber.module_attributes import Module_attribute
